@@ -109,52 +109,6 @@ class TestUndercloud(BaseTestCase):
         mock_die_tuskar_die.assert_called_once()
         mock_run_validation_groups.assert_called_once()
 
-    @mock.patch('instack_undercloud.undercloud._handle_upgrade_fact')
-    @mock.patch('instack_undercloud.undercloud._configure_logging')
-    @mock.patch('instack_undercloud.undercloud._validate_configuration')
-    @mock.patch('instack_undercloud.undercloud._run_command')
-    @mock.patch('instack_undercloud.undercloud._post_config')
-    @mock.patch('instack_undercloud.undercloud._run_orc')
-    @mock.patch('instack_undercloud.undercloud._run_yum_update')
-    @mock.patch('instack_undercloud.undercloud._run_yum_clean_all')
-    @mock.patch('instack_undercloud.undercloud._run_instack')
-    @mock.patch('instack_undercloud.undercloud._generate_environment')
-    @mock.patch('instack_undercloud.undercloud._load_config')
-    @mock.patch('instack_undercloud.undercloud._die_tuskar_die')
-    @mock.patch('instack_undercloud.undercloud._run_validation_groups')
-    def test_install_upgrade_hieradata(self, mock_run_validation_groups,
-                                       mock_die_tuskar_die, mock_load_config,
-                                       mock_generate_environment,
-                                       mock_run_instack,
-                                       mock_run_yum_clean_all,
-                                       mock_run_yum_update, mock_run_orc,
-                                       mock_post_config, mock_run_command,
-                                       mock_validate_configuration,
-                                       mock_configure_logging,
-                                       mock_upgrade_fact):
-        conf = config_fixture.Config()
-        self.useFixture(conf)
-        conf.config(hieradata_override='override.yaml')
-        with open(os.path.expanduser('~/override.yaml'), 'w') as f:
-            f.write('Something\n')
-        fake_env = mock.MagicMock()
-        mock_generate_environment.return_value = fake_env
-        undercloud.install('.', upgrade=True)
-        self.assertTrue(mock_validate_configuration.called)
-        mock_generate_environment.assert_called_with('.')
-        mock_run_instack.assert_called_with(fake_env)
-        mock_run_orc.assert_called_with(fake_env)
-        mock_run_command.assert_called_with(
-            ['sudo', 'rm', '-f', '/tmp/svc-map-services'], None, 'rm')
-        self.assertNotIn(
-            mock.call(
-                ['sudo', 'cp', 'override.yaml',
-                 '/etc/puppet/hieradata/override.yaml']),
-            mock_run_command.mock_calls)
-        mock_upgrade_fact.assert_called_with(True)
-        mock_die_tuskar_die.assert_called_once()
-        mock_run_validation_groups.assert_called_once()
-
     @mock.patch('instack_undercloud.undercloud._configure_logging')
     def test_install_exception(self, mock_configure_logging):
         mock_configure_logging.side_effect = RuntimeError('foo')
@@ -249,8 +203,7 @@ class TestCheckHostname(BaseTestCase):
             undercloud._check_hostname()
             mock_run_command.assert_called_with([
                 'sudo', '/bin/bash', '-c',
-                'sed -i "s/127.0.0.1\\(\\s*\\)/'
-                '127.0.0.1\\1test.hostname test /" '
+                'sed -i "s/127.0.0.1\(\s*\)/127.0.0.1\\1test.hostname test /" '
                 '/etc/hosts'],
                 name='hostname-to-etc-hosts')
 
@@ -267,8 +220,7 @@ class TestCheckHostname(BaseTestCase):
             undercloud._check_hostname()
             mock_run_command.assert_called_with([
                 'sudo', '/bin/bash', '-c',
-                'sed -i "s/127.0.0.1\\(\\s*\\)/'
-                '127.0.0.1\\1test.hostname test /" '
+                'sed -i "s/127.0.0.1\(\s*\)/127.0.0.1\\1test.hostname test /" '
                 '/etc/hosts'],
                 name='hostname-to-etc-hosts')
 
@@ -287,7 +239,7 @@ class TestCheckHostname(BaseTestCase):
             undercloud._check_hostname()
         mock_run_command.assert_called_with([
             'sudo', '/bin/bash', '-c',
-            'sed -i "s/127.0.0.1\\(\\s*\\)/'
+            'sed -i "s/127.0.0.1\(\s*\)/'
             '127.0.0.1\\1test-hostname.domain test-hostname /" '
             '/etc/hosts'],
             name='hostname-to-etc-hosts')
@@ -526,15 +478,6 @@ class TestGenerateEnvironment(BaseTestCase):
         self.assertEqual('http://foo/bar',
                          env['DOCKER_REGISTRY_MIRROR'])
 
-    def test_docker_insecure_registries(self):
-        conf = config_fixture.Config()
-        self.useFixture(conf)
-        conf.config(docker_insecure_registries=['http://foo/bar:8787'])
-        env = undercloud._generate_environment('.')
-        insecure_registries = json.loads(env['DOCKER_INSECURE_REGISTRIES'])
-        # Spot check one service
-        self.assertEqual(['http://foo/bar:8787'], insecure_registries)
-
     def test_generate_endpoints(self):
         env = undercloud._generate_environment('.')
         endpoint_vars = {k: v for (k, v) in env.items()
@@ -596,18 +539,10 @@ class TestGenerateEnvironment(BaseTestCase):
     def test_relative_cert_path(self):
         conf = config_fixture.Config()
         self.useFixture(conf)
-        [cert] = self.create_tempfiles([('test', 'foo')], '.pem')
-        rel_cert = os.path.basename(cert)
-        cert_path = os.path.dirname(cert)
-        cur_dir = os.getcwd()
-        try:
-            os.chdir(cert_path)
-            conf.config(undercloud_service_certificate=rel_cert)
-            env = undercloud._generate_environment('.')
-            self.assertEqual(os.path.join(os.getcwd(), rel_cert),
-                             env['UNDERCLOUD_SERVICE_CERTIFICATE'])
-        finally:
-            os.chdir(cur_dir)
+        conf.config(undercloud_service_certificate='test.pem')
+        env = undercloud._generate_environment('.')
+        self.assertEqual(os.path.join(os.getcwd(), 'test.pem'),
+                         env['UNDERCLOUD_SERVICE_CERTIFICATE'])
 
     def test_no_cert_path(self):
         env = undercloud._generate_environment('.')
@@ -757,8 +692,11 @@ class TestConfigureSshKeys(base.BaseTestCase):
 
 
 class TestPostConfig(base.BaseTestCase):
+    @mock.patch('os_client_config.make_client')
+    @mock.patch('instack_undercloud.undercloud._migrate_to_convergence')
     @mock.patch('instack_undercloud.undercloud._ensure_node_resource_classes')
     @mock.patch('instack_undercloud.undercloud._member_role_exists')
+    @mock.patch('instack_undercloud.undercloud._get_session')
     @mock.patch('ironicclient.client.get_client', autospec=True)
     @mock.patch('novaclient.client.Client', autospec=True)
     @mock.patch('swiftclient.client.Connection', autospec=True)
@@ -773,7 +711,9 @@ class TestPostConfig(base.BaseTestCase):
                          mock_configure_ssh_keys, mock_get_auth_values,
                          mock_copy_stackrc, mock_delete, mock_mistral_client,
                          mock_swift_client, mock_nova_client, mock_ir_client,
-                         mock_member_role_exists, mock_resource_classes):
+                         mock_get_session, mock_member_role_exists,
+                         mock_resource_classes, mock_migrate_to_convergence,
+                         mock_make_client):
         instack_env = {
             'UNDERCLOUD_ENDPOINT_MISTRAL_PUBLIC':
                 'http://192.168.24.1:8989/v2',
@@ -782,6 +722,7 @@ class TestPostConfig(base.BaseTestCase):
                                              'http://bletchley:5000/')
         mock_instance_nova = mock.Mock()
         mock_nova_client.return_value = mock_instance_nova
+        mock_get_session.return_value = mock.MagicMock()
         mock_instance_swift = mock.Mock()
         mock_swift_client.return_value = mock_instance_swift
         mock_instance_mistral = mock.Mock()
@@ -794,11 +735,12 @@ class TestPostConfig(base.BaseTestCase):
         flavors[0].name = 'baremetal'
         flavors[1].name = 'ceph-storage'
         mock_instance_nova.flavors.list.return_value = flavors
+        mock_heat = mock.Mock()
+        mock_make_client.return_value = mock_heat
 
-        undercloud._post_config(instack_env)
+        undercloud._post_config(instack_env, True)
         mock_nova_client.assert_called_with(
-            2, 'aturing', '3nigma', project_name='hut8',
-            auth_url='http://bletchley:5000/')
+            2, session=mock_get_session.return_value)
         self.assertTrue(mock_copy_stackrc.called)
         mock_configure_ssh_keys.assert_called_with(mock_instance_nova)
         calls = [mock.call(mock_instance_nova, flavors[0], 'baremetal', None),
@@ -815,6 +757,7 @@ class TestPostConfig(base.BaseTestCase):
         mock_resource_classes.assert_called_once_with(mock_instance_ironic)
         mock_post_config_mistral.assert_called_once_with(
             instack_env, mock_instance_mistral, mock_instance_swift)
+        mock_migrate_to_convergence.assert_called_once_with(mock_heat)
 
     @mock.patch('instack_undercloud.undercloud._get_auth_values')
     @mock.patch('instack_undercloud.undercloud._get_session')
@@ -884,10 +827,11 @@ class TestPostConfig(base.BaseTestCase):
 
         undercloud._create_default_plan(mock_mistral, [])
         mock_mistral.executions.create.assert_called_once_with(
-            'tripleo.plan_management.v1.create_default_deployment_plan',
+            'tripleo.plan_management.v1.create_deployment_plan',
             workflow_input={
                 'container': 'overcloud',
-                'queue_name': mock.ANY
+                'queue_name': mock.ANY,
+                'use_default_templates': True,
             }
         )
 
@@ -902,45 +846,26 @@ class TestPostConfig(base.BaseTestCase):
             ks_exceptions.NotFound)
 
         env = {
-            "UNDERCLOUD_DB_PASSWORD": "root-db-pass",
             "UNDERCLOUD_CEILOMETER_SNMPD_PASSWORD": "snmpd-pass"
         }
+        json_string = '{"undercloud_ceilometer_snmpd_password": "snmpd-pass"}'
 
-        json_string = {
-            "undercloud_db_password": "root-db-pass",
-            "undercloud_ceilometer_snmpd_password": "snmpd-pass"
-        }
-
-        undercloud._create_mistral_config_environment(json.loads(
-            json.dumps(env, sort_keys=True)), mock_mistral)
+        undercloud._create_mistral_config_environment(env, mock_mistral)
 
         mock_mistral.environments.create.assert_called_once_with(
-            name='tripleo.undercloud-config',
-            description='Undercloud configuration parameters',
-            variables=json.dumps(json_string, sort_keys=True))
+            name="tripleo.undercloud-config",
+            variables=json_string)
 
     def test_create_config_environment_existing(self):
         mock_mistral = mock.Mock()
-        environment = collections.namedtuple('environment',
-                                             ['name', 'variables'])
-
-        json_string = {
-            "undercloud_db_password": "root-db-pass",
-            "undercloud_ceilometer_snmpd_password": "snmpd-pass"
-        }
-
+        environment = collections.namedtuple('environment', ['name'])
         mock_mistral.environments.get.return_value = environment(
-            name='tripleo.undercloud-config',
-            variables=json.loads(json.dumps(json_string, sort_keys=True))
-           )
-
+            name='overcloud')
         env = {
-            "UNDERCLOUD_CEILOMETER_SNMPD_PASSWORD": "snmpd-pass",
-            "UNDERCLOUD_DB_PASSWORD": "root-db-pass"
+            "UNDERCLOUD_CEILOMETER_SNMPD_PASSWORD": "snmpd-pass"
         }
 
-        undercloud._create_mistral_config_environment(json.loads(
-            json.dumps(env, sort_keys=True)), mock_mistral)
+        undercloud._create_mistral_config_environment(env, mock_mistral)
         mock_mistral.executions.create.assert_not_called()
 
     def test_prepare_ssh_environment(self):
@@ -1127,6 +1052,28 @@ class TestPostConfig(base.BaseTestCase):
             '1', [{'path': '/resource_class', 'op': 'add',
                    'value': 'baremetal'}])
 
+    @mock.patch('instack_undercloud.undercloud._run_command')
+    def test_migrate_to_convergence(self, mock_run_command):
+        stacks = [mock.Mock(id='1'), mock.Mock(id='2')]
+        mock_heat = mock.Mock()
+        mock_heat.stacks.list.return_value = stacks
+        undercloud._migrate_to_convergence(mock_heat)
+        self.assertEqual([mock.call(['sudo', 'heat-manage',
+                                     'migrate_convergence_1', '1'],
+                                    name='heat-manage'),
+                          mock.call(['sudo', 'heat-manage',
+                                     'migrate_convergence_1', '2'],
+                                    name='heat-manage')],
+                         mock_run_command.mock_calls)
+
+    @mock.patch('instack_undercloud.undercloud._run_command')
+    def test_migrate_to_convergence_no_stacks(self, mock_run_command):
+        stacks = []
+        mock_heat = mock.Mock()
+        mock_heat.stacks.list.return_value = stacks
+        undercloud._migrate_to_convergence(mock_heat)
+        mock_run_command.assert_not_called()
+
     @mock.patch('instack_undercloud.undercloud._extract_from_stackrc')
     @mock.patch('instack_undercloud.undercloud._run_command')
     def test_get_auth_values(self, mock_run, mock_extract):
