@@ -23,7 +23,6 @@ import json
 import logging
 import os
 import platform
-import re
 import socket
 import subprocess
 import sys
@@ -39,9 +38,9 @@ from keystoneclient import discover
 import keystoneauth1.identity.generic as ks_auth
 from mistralclient.api import base as mistralclient_base
 from mistralclient.api import client as mistralclient
-import novaclient as nc
 from novaclient import client as novaclient
 from novaclient import exceptions
+import os_client_config
 from oslo_config import cfg
 from oslo_utils import netutils
 import psutil
@@ -354,11 +353,6 @@ _opts = [
                 help=('Whether to enable docker container images to be build '
                       'on the undercloud.')
                 ),
-    cfg.ListOpt('docker_insecure_registries',
-                default=[],
-                help=('Array of host/port combiniations of docker insecure '
-                      'registries.')
-                ),
     cfg.StrOpt('ipa_otp',
                default='',
                help=('One Time Password to register Undercloud node with '
@@ -398,7 +392,7 @@ _opts = [
 # Passwords, tokens, hashes
 _auth_opts = [
     cfg.StrOpt('undercloud_db_password',
-               help=('Password used for MySQL root user. '
+               help=('Password used for MySQL databases. '
                      'If left unset, one will be automatically generated.')
                ),
     cfg.StrOpt('undercloud_admin_token',
@@ -576,15 +570,6 @@ def _run_command(args, env=None, name=None):
     """
     if name is None:
         name = args[0]
-
-    if env is None:
-        env = os.environ
-    env = env.copy()
-
-    # When running a localized python script, we need to tell it that we're
-    # using utf-8 for stdout, otherwise it can't tell because of the pipe.
-    env['PYTHONIOENCODING'] = 'utf8'
-
     try:
         return subprocess.check_output(args,
                                        stderr=subprocess.STDOUT,
@@ -603,15 +588,6 @@ def _run_live_command(args, env=None, name=None):
     """
     if name is None:
         name = args[0]
-
-    if env is None:
-        env = os.environ
-    env = env.copy()
-
-    # When running a localized python script, we need to tell it that we're
-    # using utf-8 for stdout, otherwise it can't tell because of the pipe.
-    env['PYTHONIOENCODING'] = 'utf8'
-
     process = subprocess.Popen(args, env=env,
                                stdout=subprocess.PIPE,
                                stderr=subprocess.STDOUT)
@@ -931,7 +907,7 @@ def _generate_endpoints(instack_env):
             {'host': internal_host, 'port': 8041}),
         ('panko',
             '%s://%s:%d',
-            {'host': public_host, 'port': 8977, 'ssl_port': 13977},
+            {'host': public_host, 'port': 8977, 'ssl_port': 13779},
             {'host': internal_host, 'port': 8977}),
         ('mistral',
             '%s://%s:%d/v2',
@@ -1227,21 +1203,15 @@ def _generate_environment(instack_root):
 
     instack_env['SYSCTL_SETTINGS'] = _generate_sysctl_settings()
 
+    if CONF.docker_registry_mirror:
+        instack_env['DOCKER_REGISTRY_MIRROR'] = CONF.docker_registry_mirror
+
     instack_env['PUBLIC_INTERFACE_IP'] = instack_env['LOCAL_IP']
     instack_env['LOCAL_IP'] = instack_env['LOCAL_IP'].split('/')[0]
     instack_env['LOCAL_IP_WRAPPED'] = _wrap_ipv6(instack_env['LOCAL_IP'])
-
-    if CONF.docker_registry_mirror:
-        instack_env['DOCKER_REGISTRY_MIRROR'] = CONF.docker_registry_mirror
-    if CONF.docker_insecure_registries:
-        instack_env['DOCKER_INSECURE_REGISTRIES'] = json.dumps(
-                CONF.docker_insecure_registries)
-    else:
-        # For backward compatibility with previous defaults
-        instack_env['DOCKER_INSECURE_REGISTRIES'] = json.dumps(
-                [instack_env['LOCAL_IP'] + ':' + '8787',
-                 CONF.undercloud_admin_host + ':' + '8787'])
-
+    if instack_env['UNDERCLOUD_SERVICE_CERTIFICATE']:
+        instack_env['UNDERCLOUD_SERVICE_CERTIFICATE'] = os.path.abspath(
+            instack_env['UNDERCLOUD_SERVICE_CERTIFICATE'])
     # We're not in a chroot so this doesn't make sense, and it causes weird
     # errors if it's set.
     if instack_env.get('DIB_YUM_REPO_CONF'):
@@ -1264,19 +1234,6 @@ def _generate_environment(instack_root):
         public_host = CONF.undercloud_public_host
         instack_env['UNDERCLOUD_SERVICE_CERTIFICATE'] = (
             '/etc/pki/tls/certs/undercloud-%s.pem' % public_host)
-    elif instack_env['UNDERCLOUD_SERVICE_CERTIFICATE']:
-        raw_value = instack_env['UNDERCLOUD_SERVICE_CERTIFICATE']
-        abs_cert = os.path.abspath(raw_value)
-        if abs_cert != raw_value:
-            home_dir = os.path.expanduser('~')
-            if os.getcwd() != home_dir and os.path.exists(abs_cert):
-                LOG.warning('Using undercloud_service_certificate from '
-                            'current directory, please use an absolute path '
-                            'to remove ambiguity')
-                instack_env['UNDERCLOUD_SERVICE_CERTIFICATE'] = abs_cert
-            else:
-                instack_env['UNDERCLOUD_SERVICE_CERTIFICATE'] = os.path.join(
-                    home_dir, raw_value)
 
     return instack_env
 
@@ -1300,20 +1257,12 @@ def _generate_init_data(instack_env):
     context = instack_env.copy()
 
     if CONF.hieradata_override:
-        data_file = CONF.hieradata_override
-        hiera_entry = os.path.splitext(os.path.basename(data_file))[0]
+        hiera_entry = os.path.splitext(
+            os.path.basename(CONF.hieradata_override))[0]
         dst = os.path.join('/etc/puppet/hieradata',
-                           os.path.basename(data_file))
-        if os.path.abspath(CONF.hieradata_override) != data_file:
-            # If we don't have an absolute path, compute it
-            data_file = os.path.join(os.path.expanduser('~'), data_file)
-
-        if not os.path.exists(data_file):
-            raise RuntimeError(
-                "Could not find hieradata_override file '%s'" % data_file)
-
+                           os.path.basename(CONF.hieradata_override))
         _run_command(['sudo', 'mkdir', '-p', '/etc/puppet/hieradata'])
-        _run_command(['sudo', 'cp', data_file, dst])
+        _run_command(['sudo', 'cp', CONF.hieradata_override, dst])
         _run_command(['sudo', 'chmod', '0644', dst])
     else:
         hiera_entry = ''
@@ -1370,27 +1319,6 @@ def _run_yum_update(instack_env):
     LOG.info('yum-update completed successfully')
 
 
-def _get_ovs_interfaces():
-    interfaces = glob.glob('/etc/sysconfig/network-scripts/ifcfg-*')
-    pattern = "OVSIntPort"
-    ovs_interfaces = []
-    for interface in interfaces:
-        with open(interface, "r") as text:
-            for line in text:
-                if re.findall(pattern, line):
-                    # FIXME (holser). It might be better to get interface from
-                    # DEVICE rather than name of file.
-                    ovs_interfaces.append(interface.split('-')[-1])
-    return ovs_interfaces
-
-
-def _run_restore_ovs_interfaces(interfaces):
-    for interface in interfaces:
-        LOG.info('Running restart OVS interface %s', interface)
-        _run_command(['sudo', 'ifup', interface])
-        LOG.info('Restart OVS interface %s completed successfully', interface)
-
-
 def _run_orc(instack_env):
     args = ['sudo', 'os-refresh-config']
     LOG.info('Running os-refresh-config')
@@ -1447,32 +1375,6 @@ def _configure_ssh_keys(nova):
             nova.keypairs.create('default', pubkey.read().rstrip())
 
 
-def _ensure_ssh_selinux_permission():
-    ssh_path = os.path.expanduser('~/.ssh')
-    try:
-        enforcing = _run_command(['getenforce'])
-        if os.path.isdir(ssh_path):
-            if 'Enforcing' in enforcing:
-                file_perms = _run_command(
-                    ['find', ssh_path, '-exec', 'ls', '-lZ', '{}', ';'])
-                wrong_perm = False
-                for line in file_perms.splitlines():
-                    if 'ssh_home_t' not in line:
-                        wrong_perm = True
-                        break
-                if wrong_perm:
-                    cmd = ['semanage',
-                           'fcontext', '-a', '-t', 'ssh_home_t',
-                           "{}(/.*)?".format(ssh_path)]
-                    _run_command(cmd)
-                    _run_command(['restorecon', '-R', ssh_path])
-    except OSError as e:
-        if e.errno == os.errno.ENOENT:
-            LOG.debug("Not a SeLinux platform")
-        else:
-            raise
-
-
 def _delete_default_flavors(nova):
     """Delete the default flavors from Nova
 
@@ -1527,7 +1429,7 @@ def _ensure_flavor(nova, existing, name, profile=None):
 
 
 def _ensure_node_resource_classes(ironic):
-    for node in ironic.node.list(limit=0, fields=['uuid', 'resource_class']):
+    for node in ironic.node.list(limit=-1, fields=['uuid', 'resource_class']):
         if node.resource_class:
             if node.resource_class != DEFAULT_NODE_RESOURCE_CLASS:
                 LOG.warning('Node %s is using a resource class %s instead '
@@ -1567,51 +1469,19 @@ def _clean_os_collect_config():
 
 
 def _create_mistral_config_environment(instack_env, mistral):
-    # Store all the required passwords from the Undercloud
-    # in a Mistral environment so they can be accessed
+    # Store the snmpd password in a Mistral environment so it can be accessed
     # by the Mistral actions.
+    snmpd_password = instack_env["UNDERCLOUD_CEILOMETER_SNMPD_PASSWORD"]
 
-    config_data = {
-        'undercloud_ceilometer_snmpd_password':
-            instack_env['UNDERCLOUD_CEILOMETER_SNMPD_PASSWORD'],
-        'undercloud_db_password':
-            instack_env['UNDERCLOUD_DB_PASSWORD']
-    }
-    env_name = 'tripleo.undercloud-config'
+    env_name = "tripleo.undercloud-config"
     try:
-        env_data = mistral.environments.get(env_name).variables
+        mistral.environments.get(env_name)
     except ks_exceptions.NotFound:
-        # If the environment is not created, we need to
-        # create it with the information in config_data
         mistral.environments.create(
             name=env_name,
-            description='Undercloud configuration parameters',
-            variables=json.dumps(config_data, sort_keys=True)
-        )
-        return
-
-    # If we are upgrading from an environment without
-    # variables defined in config_data, we need to update
-    # the environment variables.
-
-    for var, value in iter(config_data.items()):
-        if var in env_data:
-            if env_data[var] != config_data[var]:
-                # Value in config_data is different
-                # need to update
-                env_data[var] = value
-        else:
-            # The value in config_data
-            # is new, we need to add it
-            env_data[var] = value
-
-    # Here we update the current environment
-    # with the variables updated
-    mistral.environments.update(
-        name=env_name,
-        description='Undercloud configuration parameters',
-        variables=json.dumps(env_data, sort_keys=True)
-    )
+            variables=json.dumps({
+                "undercloud_ceilometer_snmpd_password": snmpd_password
+            }))
 
 
 def _migrate_plans(mistral, swift, plans):
@@ -1716,8 +1586,12 @@ def _create_default_plan(mistral, plans, timeout=360):
         return
 
     execution = mistral.executions.create(
-        'tripleo.plan_management.v1.create_default_deployment_plan',
-        workflow_input={'container': plan_name, 'queue_name': queue_name}
+        'tripleo.plan_management.v1.create_deployment_plan',
+        workflow_input={
+            'container': plan_name,
+            'queue_name': queue_name,
+            'use_default_templates': True,
+        }
     )
     timeout_at = time.time() + timeout
     fail_message = ("error creating the default Deployment Plan %s "
@@ -1791,22 +1665,31 @@ def _post_config_mistral(instack_env, mistral, swift):
         _prepare_ssh_environment(mistral)
 
 
-def _post_config(instack_env):
+def _migrate_to_convergence(heat):
+    """Migrate all active stacks to use the convergence engine
+
+    This appears to be a noop if the stack has already been migrated, so it
+    should be safe to run unconditionally.
+
+    :param heat: A heat client instance
+    """
+    for stack in heat.stacks.list():
+        LOG.info('Migrating stack "%s" to convergence engine', stack.id)
+        args = ['sudo', 'heat-manage', 'migrate_convergence_1', stack.id]
+        _run_command(args, name='heat-manage')
+        LOG.info('Finished migrating stack "%s"', stack.id)
+
+
+def _post_config(instack_env, upgrade):
     _copy_stackrc()
     user, password, project, auth_url = _get_auth_values()
     sess = _get_session()
-    # TODO(andreykurilin): remove this check with support of novaclient 6.0.0
-    if nc.__version__[0] == "6":
-        nova = novaclient.Client(2, user, password, project, auth_url=auth_url)
-    else:
-        nova = novaclient.Client(2, user, password, auth_url=auth_url,
-                                 project_name=project)
+    nova = novaclient.Client(2, session=sess)
 
     ironic = ir_client.get_client(1, session=sess,
                                   os_ironic_api_version='1.21')
 
     _configure_ssh_keys(nova)
-    _ensure_ssh_selinux_permission()
     _delete_default_flavors(nova)
 
     _ensure_node_resource_classes(ironic)
@@ -1831,6 +1714,19 @@ def _post_config(instack_env):
     _post_config_mistral(instack_env, mistral, swift)
     _member_role_exists()
 
+    # NOTE(bnemec): We are turning on the convergence engine in Queens, so we
+    # need to migrate all existing stacks on upgrade.  This functionality can
+    # be removed in Rocky as all stacks should have been migrated by then.
+    if upgrade:
+        heat = os_client_config.make_client('orchestration',
+                                            auth_url=auth_url,
+                                            username=user,
+                                            password=password,
+                                            project_name=project,
+                                            project_domain_name='Default',
+                                            user_domain_name='Default')
+        _migrate_to_convergence(heat)
+
 
 def _handle_upgrade_fact(upgrade=False):
     """Create an upgrade fact for use in puppet
@@ -1889,8 +1785,13 @@ def install(instack_root, upgrade=False):
         _validate_configuration()
         instack_env = _generate_environment(instack_root)
         _generate_init_data(instack_env)
-        ovs_interfaces = _get_ovs_interfaces()
         if upgrade:
+            # We didn't complete the M->N upgrades correctly with a
+            # `nova-manage db online_data_migrations` command before. This
+            # could cause the post-upgrade db sync to fail. Better be safe
+            # than sorry and run it before package upgrade.
+            _run_command(['sudo', '/usr/bin/nova-manage', 'db',
+                          'online_data_migrations'])
             # Even if we backport https://review.openstack.org/#/c/457478/
             # into stable branches of puppet-ironic, we still need a way
             # to handle existing deployments.
@@ -1902,18 +1803,11 @@ def install(instack_root, upgrade=False):
             _die_tuskar_die()
         if CONF.undercloud_update_packages:
             _run_yum_clean_all(instack_env)
-            if ovs_interfaces:
-                _run_restore_ovs_interfaces(ovs_interfaces)
             _run_yum_update(instack_env)
         _handle_upgrade_fact(upgrade)
         _run_instack(instack_env)
         _run_orc(instack_env)
-        # FIXME (holser). The RC of issue is in OVS flow restore. Once
-        # 'systemctl reload openvswitch' is fixed ovs port restoration can be
-        # removed.
-        if ovs_interfaces:
-            _run_restore_ovs_interfaces(ovs_interfaces)
-        _post_config(instack_env)
+        _post_config(instack_env, upgrade)
         _run_command(['sudo', 'rm', '-f', '/tmp/svc-map-services'], None, 'rm')
         if upgrade and CONF.enable_validations:  # Run post-upgrade validations
             mistral_url = instack_env['UNDERCLOUD_ENDPOINT_MISTRAL_PUBLIC']
@@ -1942,17 +1836,6 @@ def pre_upgrade():
     _run_live_command(args, name='systemctl stop')
     LOG.info('Services stopped successfully')
 
-    # Ensure nova data migrations are complete before upgrading packages
-    LOG.info('Running Nova online data migration')
-    _run_command(['sudo', '-E', '/usr/bin/nova-manage', 'db',
-                  'online_data_migrations'])
-    LOG.info('Nova online data migration completed')
-
-    args = ['sudo', 'yum', 'install', '-y', 'ansible-pacemaker']
-    LOG.info('Installing Ansible Pacemaker module')
-    _run_live_command(args, name='install ansible')
-    LOG.info('Ansible pacemaker install completed successfully')
-
     args = ['sudo', 'yum', 'update', '-y']
     LOG.info('Updating full system')
     _run_live_command(args, name='yum update')
